import http
import os

import pytest

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException


def test_create_unauthorized(kernel_client_user: KernelClient, kernels_cleanup_after):
    with pytest.raises(CustomApiException) as exc:
        kernel_client_user.create_kernel(
            kernel_id="my-first-kernel",
            kernel_type=KernelType.TYPE_PYTHON,
            image="something",
        )
    assert exc.value.status == http.HTTPStatus.FORBIDDEN

@pytest.mark.skip(reason="auth disabled for now")
def test_get_unauthorized(
        kernel_client_super_admin: KernelClient,
        kernel_client_user: KernelClient,
        kernels_cleanup_after,
):
    k = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    kernel_client_user.get_kernel(kernel_id=k.kernel_id)
    # TODO enable test
    # with pytest.raises(CustomApiException) as exc:
    #     kernel_client_user.get_kernel(kernel_id=k.kernel_id)
    # assert exc.value.status == http.HTTPStatus.FORBIDDEN


def test_list_unauthorized(kernel_client_user: KernelClient, kernels_cleanup_after):
    kernel_client_user.list_kernels()
    # # TODO enable test
    # with pytest.raises(CustomApiException) as exc:
    #     kernel_client_user.list_kernels()
    # assert exc.value.status == http.HTTPStatus.FORBIDDEN

def test_update_unauthorized(
        kernel_client_super_admin: KernelClient,
        kernel_client_user: KernelClient,
        kernels_cleanup_after,
):
    k = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    with pytest.raises(CustomApiException) as exc:
        kernel_client_user.update_kernel(kernel=k)
    assert exc.value.status == http.HTTPStatus.FORBIDDEN

def test_delete_unauthorized(
        kernel_client_super_admin: KernelClient,
        kernel_client_user: KernelClient,
        kernels_cleanup_after,
):
    k = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    with pytest.raises(CustomApiException) as exc:
        kernel_client_user.delete_kernel(kernel_id=k.kernel_id)
    assert exc.value.status == http.HTTPStatus.FORBIDDEN



import os
import time
from typing import Callable
from typing import Optional

import h2o_authn
import h2o_discovery
import h2o_drive
import IPython

# Name of the platform client in the discovery response.
PLATFORM_CLIENT_NAME = "platform"

class Session:
    def __init__(
            self,
            environment_url: Optional[str] = None,
            config_path: str = None,
            platform_token: Optional[str] = None,
            token_provider: Callable[[], str] = None,
    ) -> None:
        """Initializes session.

        :param environment_url: Override for the URL of the environment passed to the discovery service.
        :param config_path: Override path to the h2o cli config file passed to the discovery service.
        :param platform_token: Platform token. If not provided, the token will be discovered.
        :param token_provider: Token provider. If not provided, the provider will be constructed from the discovered config.
        """
        if token_provider is not None:
            # Test token refresh
            token_provider()
            self.token_provider = token_provider
            return

        # Discover connection configuration
        d = h2o_discovery.discover(environment=environment_url, config_path=config_path)

        # Get discovery object containing public URIs
        d2 = h2o_discovery.discover(environment=d.environment.h2o_cloud_environment)

        # Discover platform_token if not provided
        if not platform_token:
            platform_token = d.credentials[PLATFORM_CLIENT_NAME].refresh_token

        # Discover client id
        client_id = d.clients.get(PLATFORM_CLIENT_NAME).oauth2_client_id
        if not client_id:
            raise ConnectionError(
                "Unable to discover platform oauth2_client_id connection value."
            )

        # Initialize token provider
        token_provider = h2o_authn.TokenProvider(
            issuer_url=d.environment.issuer_url,
            client_id=client_id,
            refresh_token=platform_token,
        )
        # Test token refresh
        token_provider()

        # Initialize async token provider
        async_token_provider = h2o_authn.AsyncTokenProvider(
            issuer_url=d.environment.issuer_url,
            client_id=client_id,
            refresh_token=platform_token,
        )

        self.token_provider = token_provider
        self.async_token_provider = async_token_provider
        self.discovery = d
        self.public_discovery = d2

    def get_token_provider(self) -> Callable[[], str]:
        """Returns token provider."""
        return self.token_provider

    def get_async_token_provider(self) -> Callable[[], str]:
        """Returns asynchronous token provider."""
        return self.async_token_provider

    async def download_file(self, path: str) -> None:
        """Download file from Kernel filesystem."""
        
        # Init H2O Drive client
        drive_client = await h2o_drive.Drive(token=self.async_token_provider, endpoint_url=self.public_discovery.services["drive"].uri)

        # Prepare new space for the file
        notebook_downloads_space = drive_client.my_bucket().with_prefix("notebook-downloads/")
        object_name = f"{time.time_ns()}/{os.path.basename(path)}"

        # Upload the file
        await notebook_downloads_space.upload_file(path, object_name)

        # Generate presigned URL for the file
        url = await notebook_downloads_space.generate_presigned_url(object_name)

        # Open the URL in a new tab of the browser which downloads the file
        IPython.display.display(IPython.display.Javascript(f'window.open("{url}");'))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/notebook/v1/kernel_service.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from h2o_notebook.gen.ai.h2o.notebook.v1 import kernel_pb2 as ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import client_pb2 as google_dot_api_dot_client__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'ai/h2o/notebook/v1/kernel_service.proto\x12\x12\x61i.h2o.notebook.v1\x1a\x1f\x61i/h2o/notebook/v1/kernel.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/api/resource.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a google/protobuf/field_mask.proto\"r\n\x13\x43reateKernelRequest\x12\x38\n\x06kernel\x18\x02 \x01(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelB\x04\xe2\x41\x01\x02R\x06kernel\x12!\n\tkernel_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x02R\x08kernelId\"J\n\x14\x43reateKernelResponse\x12\x32\n\x06kernel\x18\x01 \x01(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelR\x06kernel\"G\n\x10GetKernelRequest\x12\x33\n\x04name\x18\x01 \x01(\tB\x1f\xe2\x41\x01\x02\xfa\x41\x18\n\x16notebook.h2o.ai/KernelR\x04name\"G\n\x11GetKernelResponse\x12\x32\n\x06kernel\x18\x01 \x01(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelR\x06kernel\"\\\n\x12ListKernelsRequest\x12!\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01R\x08pageSize\x12#\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01R\tpageToken\"s\n\x13ListKernelsResponse\x12\x34\n\x07kernels\x18\x01 \x03(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelR\x07kernels\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x92\x01\n\x13UpdateKernelRequest\x12\x38\n\x06kernel\x18\x01 \x01(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelB\x04\xe2\x41\x01\x02R\x06kernel\x12\x41\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x02R\nupdateMask\"J\n\x14UpdateKernelResponse\x12\x32\n\x06kernel\x18\x01 \x01(\x0b\x32\x1a.ai.h2o.notebook.v1.KernelR\x06kernel\"J\n\x13\x44\x65leteKernelRequest\x12\x33\n\x04name\x18\x01 \x01(\tB\x1f\xe2\x41\x01\x02\xfa\x41\x18\n\x16notebook.h2o.ai/KernelR\x04name\"\x16\n\x14\x44\x65leteKernelResponse2\xc6\x05\n\rKernelService\x12\x91\x01\n\x0c\x43reateKernel\x12\'.ai.h2o.notebook.v1.CreateKernelRequest\x1a(.ai.h2o.notebook.v1.CreateKernelResponse\".\xda\x41\x10kernel,kernel_id\x82\xd3\xe4\x93\x02\x15\"\x0b/v1/kernels:\x06kernel\x12}\n\tGetKernel\x12$.ai.h2o.notebook.v1.GetKernelRequest\x1a%.ai.h2o.notebook.v1.GetKernelResponse\"#\xda\x41\x04name\x82\xd3\xe4\x93\x02\x16\x12\x14/v1/{name=kernels/*}\x12s\n\x0bListKernels\x12&.ai.h2o.notebook.v1.ListKernelsRequest\x1a\'.ai.h2o.notebook.v1.ListKernelsResponse\"\x13\x82\xd3\xe4\x93\x02\r\x12\x0b/v1/kernels\x12\xa3\x01\n\x0cUpdateKernel\x12\'.ai.h2o.notebook.v1.UpdateKernelRequest\x1a(.ai.h2o.notebook.v1.UpdateKernelResponse\"@\xda\x41\x12kernel,update_mask\x82\xd3\xe4\x93\x02%2\x1b/v1/{kernel.name=kernels/*}:\x06kernel\x12\x86\x01\n\x0c\x44\x65leteKernel\x12\'.ai.h2o.notebook.v1.DeleteKernelRequest\x1a(.ai.h2o.notebook.v1.DeleteKernelResponse\"#\xda\x41\x04name\x82\xd3\xe4\x93\x02\x16*\x14/v1/{name=kernels/*}B\xd4\x01\n\x16\x63om.ai.h2o.notebook.v1B\x12KernelServiceProtoP\x01Z;github.com/h2oai/notebook/api/ai/h2o/notebook/v1;notebookv1\xa2\x02\x03\x41HN\xaa\x02\x12\x41i.H2o.Notebook.V1\xca\x02\x12\x41i\\H2o\\Notebook\\V1\xe2\x02\x1e\x41i\\H2o\\Notebook\\V1\\GPBMetadata\xea\x02\x15\x41i::H2o::Notebook::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.notebook.v1.kernel_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ai.h2o.notebook.v1B\022KernelServiceProtoP\001Z;github.com/h2oai/notebook/api/ai/h2o/notebook/v1;notebookv1\242\002\003AHN\252\002\022Ai.H2o.Notebook.V1\312\002\022Ai\\H2o\\Notebook\\V1\342\002\036Ai\\H2o\\Notebook\\V1\\GPBMetadata\352\002\025Ai::H2o::Notebook::V1'
  _globals['_CREATEKERNELREQUEST'].fields_by_name['kernel']._options = None
  _globals['_CREATEKERNELREQUEST'].fields_by_name['kernel']._serialized_options = b'\342A\001\002'
  _globals['_CREATEKERNELREQUEST'].fields_by_name['kernel_id']._options = None
  _globals['_CREATEKERNELREQUEST'].fields_by_name['kernel_id']._serialized_options = b'\342A\001\002'
  _globals['_GETKERNELREQUEST'].fields_by_name['name']._options = None
  _globals['_GETKERNELREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002\372A\030\n\026notebook.h2o.ai/Kernel'
  _globals['_LISTKERNELSREQUEST'].fields_by_name['page_size']._options = None
  _globals['_LISTKERNELSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTKERNELSREQUEST'].fields_by_name['page_token']._options = None
  _globals['_LISTKERNELSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEKERNELREQUEST'].fields_by_name['kernel']._options = None
  _globals['_UPDATEKERNELREQUEST'].fields_by_name['kernel']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEKERNELREQUEST'].fields_by_name['update_mask']._options = None
  _globals['_UPDATEKERNELREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\002'
  _globals['_DELETEKERNELREQUEST'].fields_by_name['name']._options = None
  _globals['_DELETEKERNELREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002\372A\030\n\026notebook.h2o.ai/Kernel'
  _globals['_KERNELSERVICE'].methods_by_name['CreateKernel']._options = None
  _globals['_KERNELSERVICE'].methods_by_name['CreateKernel']._serialized_options = b'\332A\020kernel,kernel_id\202\323\344\223\002\025\"\013/v1/kernels:\006kernel'
  _globals['_KERNELSERVICE'].methods_by_name['GetKernel']._options = None
  _globals['_KERNELSERVICE'].methods_by_name['GetKernel']._serialized_options = b'\332A\004name\202\323\344\223\002\026\022\024/v1/{name=kernels/*}'
  _globals['_KERNELSERVICE'].methods_by_name['ListKernels']._options = None
  _globals['_KERNELSERVICE'].methods_by_name['ListKernels']._serialized_options = b'\202\323\344\223\002\r\022\013/v1/kernels'
  _globals['_KERNELSERVICE'].methods_by_name['UpdateKernel']._options = None
  _globals['_KERNELSERVICE'].methods_by_name['UpdateKernel']._serialized_options = b'\332A\022kernel,update_mask\202\323\344\223\002%2\033/v1/{kernel.name=kernels/*}:\006kernel'
  _globals['_KERNELSERVICE'].methods_by_name['DeleteKernel']._options = None
  _globals['_KERNELSERVICE'].methods_by_name['DeleteKernel']._serialized_options = b'\332A\004name\202\323\344\223\002\026*\024/v1/{name=kernels/*}'
  _globals['_CREATEKERNELREQUEST']._serialized_start=245
  _globals['_CREATEKERNELREQUEST']._serialized_end=359
  _globals['_CREATEKERNELRESPONSE']._serialized_start=361
  _globals['_CREATEKERNELRESPONSE']._serialized_end=435
  _globals['_GETKERNELREQUEST']._serialized_start=437
  _globals['_GETKERNELREQUEST']._serialized_end=508
  _globals['_GETKERNELRESPONSE']._serialized_start=510
  _globals['_GETKERNELRESPONSE']._serialized_end=581
  _globals['_LISTKERNELSREQUEST']._serialized_start=583
  _globals['_LISTKERNELSREQUEST']._serialized_end=675
  _globals['_LISTKERNELSRESPONSE']._serialized_start=677
  _globals['_LISTKERNELSRESPONSE']._serialized_end=792
  _globals['_UPDATEKERNELREQUEST']._serialized_start=795
  _globals['_UPDATEKERNELREQUEST']._serialized_end=941
  _globals['_UPDATEKERNELRESPONSE']._serialized_start=943
  _globals['_UPDATEKERNELRESPONSE']._serialized_end=1017
  _globals['_DELETEKERNELREQUEST']._serialized_start=1019
  _globals['_DELETEKERNELREQUEST']._serialized_end=1093
  _globals['_DELETEKERNELRESPONSE']._serialized_start=1095
  _globals['_DELETEKERNELRESPONSE']._serialized_end=1117
  _globals['_KERNELSERVICE']._serialized_start=1120
  _globals['_KERNELSERVICE']._serialized_end=1830
# @@protoc_insertion_point(module_scope)



import http
import os

import pytest

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException


@pytest.mark.parametrize(
    ["page_size", "page_token"],
    [
        (-20, ""),
        (0, "non-existing-token"),
    ],
)
def test_list_validation(
        kernel_client_super_admin: KernelClient, page_size, page_token
):
    with pytest.raises(CustomApiException) as exc:
        kernel_client_super_admin.list_kernels(
            page_size=page_size,
            page_token=page_token,
        )
    assert exc.value.status == http.HTTPStatus.BAD_REQUEST

def test_list(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    # Test no profiles found.
    page = kernel_client_super_admin.list_kernels()
    assert len(page.kernels) == 0
    assert page.next_page_token == ""

    # Arrange
    kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )
    kernel_client_super_admin.create_kernel(
        kernel_id="my-second-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )
    kernel_client_super_admin.create_kernel(
        kernel_id="my-third-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    # Test getting first page.
    page = kernel_client_super_admin.list_kernels(page_size=1)
    assert len(page.kernels) == 1
    assert page.next_page_token != ""

    # Test getting second page.
    page = kernel_client_super_admin.list_kernels(
        page_size=1, page_token=page.next_page_token
    )
    assert len(page.kernels) == 1
    assert page.next_page_token != ""

    # Test getting last page.
    page = kernel_client_super_admin.list_kernels(
        page_size=1, page_token=page.next_page_token
    )
    assert len(page.kernels) == 1
    assert page.next_page_token == ""

    # Test exceeding max page size.
    page = kernel_client_super_admin.list_kernels(page_size=1001)
    assert len(page.kernels) == 3
    assert page.next_page_token == ""

def test_list_all(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    # Arrange
    kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )
    kernel_client_super_admin.create_kernel(
        kernel_id="my-second-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )
    kernel_client_super_admin.create_kernel(
        kernel_id="my-third-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    # Test basic list_all.
    kernels = kernel_client_super_admin.list_all_kernels()
    assert len(kernels) == 3


from enum import Enum

from h2o_notebook.gen.model.v1_kernel_type import V1KernelType


class KernelType(Enum):
    TYPE_UNSPECIFIED = "TYPE_UNSPECIFIED"
    TYPE_PYTHON = "TYPE_PYTHON"
    TYPE_R = "TYPE_R"
    TYPE_SPARK_PYTHON = "TYPE_SPARK_PYTHON"
    TYPE_SPARK_R = "TYPE_SPARK_R"

    def to_api_object(self) -> V1KernelType:
        return V1KernelType(self.name)


def from_api_object(kernel_type: V1KernelType) -> KernelType:
    return KernelType(str(kernel_type))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/notebook/v1/kernel.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x61i/h2o/notebook/v1/kernel.proto\x12\x12\x61i.h2o.notebook.v1\x1a\x19google/api/resource.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x8e\x07\n\x06Kernel\x12\x18\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x03R\x04name\x12\'\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x01R\x0b\x64isplayName\x12\x39\n\x04type\x18\x03 \x01(\x0e\x32\x1f.ai.h2o.notebook.v1.Kernel.TypeB\x04\xe2\x41\x01\x02R\x04type\x12\x1a\n\x05image\x18\x04 \x01(\tB\x04\xe2\x41\x01\x02R\x05image\x12%\n\x0b\x63pu_request\x18\x05 \x01(\x05\x42\x04\xe2\x41\x01\x01R\ncpuRequest\x12!\n\tcpu_limit\x18\x06 \x01(\x05\x42\x04\xe2\x41\x01\x01R\x08\x63puLimit\x12\x30\n\x11gpu_resource_name\x18\x07 \x01(\tB\x04\xe2\x41\x01\x07R\x0fgpuResourceName\x12%\n\x0bgpu_request\x18\x08 \x01(\x05\x42\x04\xe2\x41\x01\x01R\ngpuRequest\x12!\n\tgpu_limit\x18\t \x01(\x05\x42\x04\xe2\x41\x01\x01R\x08gpuLimit\x12\x36\n\x14memory_bytes_request\x18\n \x01(\x03\x42\x04\xe2\x41\x01\x01R\x12memoryBytesRequest\x12\x32\n\x12memory_bytes_limit\x18\x0b \x01(\x03\x42\x04\xe2\x41\x01\x01R\x10memoryBytesLimit\x12\x39\n\x16yaml_pod_template_spec\x18\x0c \x01(\tB\x04\xe2\x41\x01\x01R\x13yamlPodTemplateSpec\x12\x41\n\x0b\x63reate_time\x18\x64 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03R\ncreateTime\x12\x1e\n\x07\x63reator\x18\x65 \x01(\tB\x04\xe2\x41\x01\x03R\x07\x63reator\x12\x46\n\x0bupdate_time\x18\x66 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03H\x00R\nupdateTime\x88\x01\x01\x12#\n\x07updater\x18g \x01(\tB\x04\xe2\x41\x01\x03H\x01R\x07updater\x88\x01\x01\"b\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0f\n\x0bTYPE_PYTHON\x10\x01\x12\n\n\x06TYPE_R\x10\x02\x12\x15\n\x11TYPE_SPARK_PYTHON\x10\x03\x12\x10\n\x0cTYPE_SPARK_R\x10\x04:-\xea\x41*\n\x16notebook.h2o.ai/Kernel\x12\x10kernels/{kernel}B\x0e\n\x0c_update_timeB\n\n\x08_updaterB\xcd\x01\n\x16\x63om.ai.h2o.notebook.v1B\x0bKernelProtoP\x01Z;github.com/h2oai/notebook/api/ai/h2o/notebook/v1;notebookv1\xa2\x02\x03\x41HN\xaa\x02\x12\x41i.H2o.Notebook.V1\xca\x02\x12\x41i\\H2o\\Notebook\\V1\xe2\x02\x1e\x41i\\H2o\\Notebook\\V1\\GPBMetadata\xea\x02\x15\x41i::H2o::Notebook::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.notebook.v1.kernel_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.ai.h2o.notebook.v1B\013KernelProtoP\001Z;github.com/h2oai/notebook/api/ai/h2o/notebook/v1;notebookv1\242\002\003AHN\252\002\022Ai.H2o.Notebook.V1\312\002\022Ai\\H2o\\Notebook\\V1\342\002\036Ai\\H2o\\Notebook\\V1\\GPBMetadata\352\002\025Ai::H2o::Notebook::V1'
  _globals['_KERNEL'].fields_by_name['name']._options = None
  _globals['_KERNEL'].fields_by_name['name']._serialized_options = b'\342A\001\003'
  _globals['_KERNEL'].fields_by_name['display_name']._options = None
  _globals['_KERNEL'].fields_by_name['display_name']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['type']._options = None
  _globals['_KERNEL'].fields_by_name['type']._serialized_options = b'\342A\001\002'
  _globals['_KERNEL'].fields_by_name['image']._options = None
  _globals['_KERNEL'].fields_by_name['image']._serialized_options = b'\342A\001\002'
  _globals['_KERNEL'].fields_by_name['cpu_request']._options = None
  _globals['_KERNEL'].fields_by_name['cpu_request']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['cpu_limit']._options = None
  _globals['_KERNEL'].fields_by_name['cpu_limit']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['gpu_resource_name']._options = None
  _globals['_KERNEL'].fields_by_name['gpu_resource_name']._serialized_options = b'\342A\001\007'
  _globals['_KERNEL'].fields_by_name['gpu_request']._options = None
  _globals['_KERNEL'].fields_by_name['gpu_request']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['gpu_limit']._options = None
  _globals['_KERNEL'].fields_by_name['gpu_limit']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['memory_bytes_request']._options = None
  _globals['_KERNEL'].fields_by_name['memory_bytes_request']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['memory_bytes_limit']._options = None
  _globals['_KERNEL'].fields_by_name['memory_bytes_limit']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['yaml_pod_template_spec']._options = None
  _globals['_KERNEL'].fields_by_name['yaml_pod_template_spec']._serialized_options = b'\342A\001\001'
  _globals['_KERNEL'].fields_by_name['create_time']._options = None
  _globals['_KERNEL'].fields_by_name['create_time']._serialized_options = b'\342A\001\003'
  _globals['_KERNEL'].fields_by_name['creator']._options = None
  _globals['_KERNEL'].fields_by_name['creator']._serialized_options = b'\342A\001\003'
  _globals['_KERNEL'].fields_by_name['update_time']._options = None
  _globals['_KERNEL'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_KERNEL'].fields_by_name['updater']._options = None
  _globals['_KERNEL'].fields_by_name['updater']._serialized_options = b'\342A\001\003'
  _globals['_KERNEL']._options = None
  _globals['_KERNEL']._serialized_options = b'\352A*\n\026notebook.h2o.ai/Kernel\022\020kernels/{kernel}'
  _globals['_KERNEL']._serialized_start=149
  _globals['_KERNEL']._serialized_end=1059
  _globals['_KERNEL_TYPE']._serialized_start=886
  _globals['_KERNEL_TYPE']._serialized_end=984
# @@protoc_insertion_point(module_scope)



import os

import grpc
from jupyter_client.kernelspec import KernelSpec
from jupyter_client.kernelspec import KernelSpecManager

from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_pb2 import Kernel
from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2 import GetKernelRequest
from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2 import ListKernelsRequest
from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2_grpc import (
    KernelServiceStub,
)


class NotebookKernelSpecManager(KernelSpecManager):
    def __init__(self, **kwargs):
        self.log.info("NotebookKernelSpecManager.__init__")
        addr = os.getenv("H2O_NOTEBOOK_SERVER_GRPC_ADDR")
        if not addr:
            raise ValueError("H2O_NOTEBOOK_SERVER_GRPC_ADDR environment variable is not set")

        channel = grpc.insecure_channel(addr)
        self.notebook_client = KernelServiceStub(channel)

        # Authorize GRPC requests with Kubernetes service account token if not disabled (local testing)
        # TODO: Support projected service account tokens (the token may update, so handle accordingly)
        self.request_metadata = []
        if os.getenv("NOTEBOOK_ENABLE_GRPC_AUTH", 'True').lower() == "false":
            self.log.info("GRPC auth is disabled")
        else:
            path = os.getenv("KUBERNETES_SERVICE_ACCOUNT_TOKEN_FILE", "/var/run/secrets/kubernetes.io/serviceaccount/token")
            self.request_metadata.append(('authorization', 'Bearer ' + read_file(path)))

        # Try calling an endpoint to verify the connection
        self.notebook_client.ListKernels(request=ListKernelsRequest(), metadata=self.request_metadata)

        super().__init__(**kwargs)

    def get_all_specs(self):
        res = {}

        next_page_token = ""
        while True:
            kernel_list = self.notebook_client.ListKernels(
                request=ListKernelsRequest(
                    page_size=0,
                    page_token=next_page_token,
                ),
                metadata=self.request_metadata
            )

            for kernel in kernel_list.kernels:
                res[kernel.name] = {
                    "resource_dir": "",
                    "spec": self.__to_kernel_spec(kernel).to_dict()
                }

            next_page_token = kernel_list.next_page_token
            if next_page_token == "":
                break

        return res

    def get_kernel_spec(self, kernel_name) -> KernelSpec:
        return self.__to_kernel_spec(self.notebook_client.GetKernel(request=GetKernelRequest(name=kernel_name), metadata=self.request_metadata).kernel)

    @staticmethod
    def __to_kernel_spec(kernel: Kernel) -> KernelSpec:
        return KernelSpec(
            display_name=kernel.display_name,
            language=NotebookKernelSpecManager.__to_language(kernel.type),
            metadata={
                "process_proxy": {
                    "class_name": "enterprise_gateway.services.processproxies.k8s.KubernetesProcessProxy",
                    "config": NotebookKernelSpecManager.__build_config(kernel.type, kernel.image)
                }
            },
            env=NotebookKernelSpecManager.__build_env(kernel),
            argv=NotebookKernelSpecManager.__build_argv(kernel.name, kernel.type),
        )

    @staticmethod
    def __to_language(kernel_type: int) -> str:
        if kernel_type in (Kernel.TYPE_PYTHON, Kernel.TYPE_SPARK_PYTHON):
            return "python"
        elif kernel_type in (Kernel.TYPE_R, Kernel.TYPE_SPARK_R):
            return "R"
        else:
            raise ValueError("Unknown kernel type: " + str(kernel_type))

    @staticmethod
    def __build_config(kernel_type: int, image:str) -> {}:
        cfg = {"image_name": image}
        if kernel_type in (Kernel.TYPE_SPARK_R, Kernel.TYPE_SPARK_PYTHON):
            cfg["executor_image_name"] = "gcr.io/vorvan/h2oai/h2o-kernel-pyspark:0.3.4"

        return cfg

    @staticmethod
    def __build_env(kernel: Kernel) -> {}:
        env = {}
        if kernel.type in (Kernel.TYPE_SPARK_R, Kernel.TYPE_SPARK_PYTHON):
            env["SPARK_HOME"] = "/opt/spark"
            env["SPARK_OPTS"] = "--master k8s://https://${KUBERNETES_SERVICE_HOST}:${KUBERNETES_SERVICE_PORT} --deploy-mode cluster --name ${KERNEL_USERNAME}-${KERNEL_ID} --conf spark.kubernetes.namespace=${KERNEL_NAMESPACE} --conf spark.kubernetes.driver.label.app=enterprise-gateway --conf spark.kubernetes.driver.label.kernel_id=${KERNEL_ID} --conf spark.kubernetes.driver.label.component=kernel --conf spark.kubernetes.executor.label.app=enterprise-gateway --conf spark.kubernetes.executor.label.kernel_id=${KERNEL_ID} --conf spark.kubernetes.executor.label.component=worker --conf spark.kubernetes.driver.container.image=${KERNEL_IMAGE} --conf spark.kubernetes.executor.container.image=${KERNEL_EXECUTOR_IMAGE} --conf spark.kubernetes.authenticate.driver.serviceAccountName=${KERNEL_SERVICE_ACCOUNT_NAME} --conf spark.kubernetes.submission.waitAppCompletion=false --conf spark.kubernetes.driverEnv.HTTP2_DISABLE=true --conf spark.scheduler.minRegisteredResourcesRatio=1 --conf spark.kubernetes.driver.annotation.cloud.h2o.ai/creator-display-name=${KERNEL_USER_NAME} --conf spark.kubernetes.driver.annotation.cloud.h2o.ai/owner-display-name=${KERNEL_USER_NAME} --conf spark.kubernetes.driver.label.cloud.h2o.ai/owner=${KERNEL_USER_SUB} --conf spark.kubernetes.driver.label.cloud.h2o.ai/creator=${KERNEL_USER_SUB} --conf spark.kubernetes.driver.label.telemetry.cloud.h2o.ai/include=true --conf spark.kubernetes.executor.annotation.cloud.h2o.ai/creator-display-name=${KERNEL_USER_NAME} --conf spark.kubernetes.executor.annotation.cloud.h2o.ai/owner-display-name=${KERNEL_USER_NAME} --conf spark.kubernetes.executor.label.cloud.h2o.ai/owner=${KERNEL_USER_SUB} --conf spark.kubernetes.executor.label.cloud.h2o.ai/creator=${KERNEL_USER_SUB} --conf spark.kubernetes.executor.label.telemetry.cloud.h2o.ai/include=true ${KERNEL_EXTRA_SPARK_OPTS}"
            env["HTTP2_DISABLE"] = "true"
            env["LAUNCH_OPTS"] = ""

        if kernel.memory_bytes_limit:
            env["KERNEL_MEMORY_LIMIT"] = str(kernel.memory_bytes_limit)
        if kernel.memory_bytes_request:
            env["KERNEL_MEMORY"] = str(kernel.memory_bytes_request)
        if kernel.cpu_limit:
            env["KERNEL_CPUS_LIMIT"] = str(kernel.cpu_limit)
        if kernel.cpu_request:
            env["KERNEL_CPUS"] = str(kernel.cpu_request)
        if kernel.gpu_limit:
            env["KERNEL_GPUS_LIMIT"] = str(kernel.gpu_limit)
        if kernel.gpu_request:
            env["KERNEL_GPUS"] = str(kernel.gpu_request)

        return env

    @staticmethod
    def __build_argv(kernel_name: str, kernel_type: int) -> []:
        if kernel_type in (Kernel.TYPE_PYTHON, Kernel.TYPE_R):
            return [
                "python",
                f"/tmp/{kernel_name}/launch_kubernetes.py",
                "--RemoteProcessProxy.kernel-id",
                "{kernel_id}",
                "--RemoteProcessProxy.port-range",
                "{port_range}",
                "--RemoteProcessProxy.response-address",
                "{response_address}",
                "--RemoteProcessProxy.public-key",
                "{public_key}"
            ]
        elif kernel_type == Kernel.TYPE_SPARK_PYTHON:
            return [
                f"/tmp/{kernel_name}/run-py.sh",
                "--RemoteProcessProxy.kernel-id",
                "{kernel_id}",
                "--RemoteProcessProxy.port-range",
                "{port_range}",
                "--RemoteProcessProxy.response-address",
                "{response_address}",
                "--RemoteProcessProxy.public-key",
                "{public_key}",
                "--RemoteProcessProxy.spark-context-initialization-mode",
                "lazy"
            ]
        elif kernel_type == Kernel.TYPE_SPARK_R:
            return [
                f"/tmp/{kernel_name}/run-r.sh",
                "--RemoteProcessProxy.kernel-id",
                "{kernel_id}",
                "--RemoteProcessProxy.port-range",
                "{port_range}",
                "--RemoteProcessProxy.response-address",
                "{response_address}",
                "--RemoteProcessProxy.public-key",
                "{public_key}",
                "--RemoteProcessProxy.spark-context-initialization-mode",
                "lazy"
            ]
        else:
            raise ValueError("Unknown kernel type: " + str(kernel_type))


def read_file(path: str) -> str:
    with open(path, "r") as f:
        return f.read()




"""
    ai/h2o/notebook/v1/kernel.proto

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: version not set
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from h2o_notebook.gen.api_client import ApiClient, Endpoint as _Endpoint
from h2o_notebook.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from h2o_notebook.gen.model.kernel_resource import KernelResource
from h2o_notebook.gen.model.rpc_status import RpcStatus
from h2o_notebook.gen.model.v1_create_kernel_response import V1CreateKernelResponse
from h2o_notebook.gen.model.v1_get_kernel_response import V1GetKernelResponse
from h2o_notebook.gen.model.v1_kernel import V1Kernel
from h2o_notebook.gen.model.v1_list_kernels_response import V1ListKernelsResponse
from h2o_notebook.gen.model.v1_update_kernel_response import V1UpdateKernelResponse


class KernelServiceApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.kernel_service_create_kernel_endpoint = _Endpoint(
            settings={
                'response_type': (V1CreateKernelResponse,),
                'auth': [],
                'endpoint_path': '/v1/kernels',
                'operation_id': 'kernel_service_create_kernel',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'kernel_id',
                    'kernel',
                ],
                'required': [
                    'kernel_id',
                    'kernel',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'kernel_id':
                        (str,),
                    'kernel':
                        (V1Kernel,),
                },
                'attribute_map': {
                    'kernel_id': 'kernelId',
                },
                'location_map': {
                    'kernel_id': 'query',
                    'kernel': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.kernel_service_delete_kernel_endpoint = _Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str, none_type,),
                'auth': [],
                'endpoint_path': '/v1/{name}',
                'operation_id': 'kernel_service_delete_kernel',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'name',
                ],
                'required': [
                    'name',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name',
                ]
            },
            root_map={
                'validations': {
                    ('name',): {

                        'regex': {
                            'pattern': r'kernels\/[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name':
                        (str,),
                },
                'attribute_map': {
                    'name': 'name',
                },
                'location_map': {
                    'name': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.kernel_service_get_kernel_endpoint = _Endpoint(
            settings={
                'response_type': (V1GetKernelResponse,),
                'auth': [],
                'endpoint_path': '/v1/{name}',
                'operation_id': 'kernel_service_get_kernel',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name',
                ],
                'required': [
                    'name',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name',
                ]
            },
            root_map={
                'validations': {
                    ('name',): {

                        'regex': {
                            'pattern': r'kernels\/[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name':
                        (str,),
                },
                'attribute_map': {
                    'name': 'name',
                },
                'location_map': {
                    'name': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.kernel_service_list_kernels_endpoint = _Endpoint(
            settings={
                'response_type': (V1ListKernelsResponse,),
                'auth': [],
                'endpoint_path': '/v1/kernels',
                'operation_id': 'kernel_service_list_kernels',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'page_size',
                    'page_token',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'page_size':
                        (int,),
                    'page_token':
                        (str,),
                },
                'attribute_map': {
                    'page_size': 'pageSize',
                    'page_token': 'pageToken',
                },
                'location_map': {
                    'page_size': 'query',
                    'page_token': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.kernel_service_update_kernel_endpoint = _Endpoint(
            settings={
                'response_type': (V1UpdateKernelResponse,),
                'auth': [],
                'endpoint_path': '/v1/{kernel.name}',
                'operation_id': 'kernel_service_update_kernel',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'kernel_name',
                    'update_mask',
                    'kernel',
                ],
                'required': [
                    'kernel_name',
                    'update_mask',
                    'kernel',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'kernel_name',
                ]
            },
            root_map={
                'validations': {
                    ('kernel_name',): {

                        'regex': {
                            'pattern': r'kernels\/[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'kernel_name':
                        (str,),
                    'update_mask':
                        (str,),
                    'kernel':
                        (KernelResource,),
                },
                'attribute_map': {
                    'kernel_name': 'kernel.name',
                    'update_mask': 'updateMask',
                },
                'location_map': {
                    'kernel_name': 'path',
                    'update_mask': 'query',
                    'kernel': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

    def kernel_service_create_kernel(
        self,
        kernel_id,
        kernel,
        **kwargs
    ):
        """Create a new Kernel. (-- api-linter: core::0133::http-uri-parent=disabled     aip.dev/not-precedent: Kernel has no parent (it's a top-level resource) --) (-- api-linter: core::0133::method-signature=disabled     aip.dev/not-precedent: Kernel has no parent (it's a top-level resource) --)  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.kernel_service_create_kernel(kernel_id, kernel, async_req=True)
        >>> result = thread.get()

        Args:
            kernel_id (str): Specify the Kernel resource ID, which will become a part of the Kernel resource name.  It must: - contain 1-63 characters - contain only lowercase alphanumeric characters or hyphen ('-') - start with an alphabetic character - end with an alphanumeric character  This matches a regular expression: ^[a-z]([a-z0-9-]{0,61}[a-z0-9])?$  (-- api-linter: core::0133::request-required-fields=disabled     aip.dev/not-precedent: Google API linter bug (kernel_id is a valid field in Create request...) --)
            kernel (V1Kernel): The Kernel resource to create. (-- api-linter: core::0133::request-required-fields=disabled     aip.dev/not-precedent: Google API linter bug (kernel is a valid field in Create request...) --)

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            V1CreateKernelResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['kernel_id'] = \
            kernel_id
        kwargs['kernel'] = \
            kernel
        return self.kernel_service_create_kernel_endpoint.call_with_http_info(**kwargs)

    def kernel_service_delete_kernel(
        self,
        name,
        **kwargs
    ):
        """Delete a Kernel.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.kernel_service_delete_kernel(name, async_req=True)
        >>> result = thread.get()

        Args:
            name (str): Kernel resource name.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            bool, date, datetime, dict, float, int, list, str, none_type
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name'] = \
            name
        return self.kernel_service_delete_kernel_endpoint.call_with_http_info(**kwargs)

    def kernel_service_get_kernel(
        self,
        name,
        **kwargs
    ):
        """Return a specific Kernel.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.kernel_service_get_kernel(name, async_req=True)
        >>> result = thread.get()

        Args:
            name (str): Kernel resource name.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            V1GetKernelResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name'] = \
            name
        return self.kernel_service_get_kernel_endpoint.call_with_http_info(**kwargs)

    def kernel_service_list_kernels(
        self,
        **kwargs
    ):
        """Return a collection of Kernels.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.kernel_service_list_kernels(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            page_size (int): Maximum number of Kernels to return in a response. If unspecified (or set to 0), at most 50 Kernels will be returned. The maximum value is 1000; values above 1000 will be coerced to 1000.. [optional]
            page_token (str): Leave unset to receive the initial page. To list any subsequent pages use the value of 'next_page_token' returned from the ListKernelsResponse.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            V1ListKernelsResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.kernel_service_list_kernels_endpoint.call_with_http_info(**kwargs)

    def kernel_service_update_kernel(
        self,
        kernel_name,
        update_mask,
        kernel,
        **kwargs
    ):
        """Update a Kernel.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.kernel_service_update_kernel(kernel_name, update_mask, kernel, async_req=True)
        >>> result = thread.get()

        Args:
            kernel_name (str): Resource name.
            update_mask (str): The list of paths referencing which fields to update. Update mask must be non-empty.  Allowed field paths are: {display_name, type, image, cpu_request, cpu_limit, gpu_resource_name, gpu_request, gpu_limit, memory_bytes_request, memory_bytes_limit, yaml_pod_template_spec}.  To update all allowed fields, specify exactly one path with value \"*\".
            kernel (KernelResource): Kernel resource.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            V1UpdateKernelResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['kernel_name'] = \
            kernel_name
        kwargs['update_mask'] = \
            update_mask
        kwargs['kernel'] = \
            kernel
        return self.kernel_service_update_kernel_endpoint.call_with_http_info(**kwargs)




from google.api import resource_pb2 as _resource_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Kernel(_message.Message):
    __slots__ = ("name", "display_name", "type", "image", "cpu_request", "cpu_limit", "gpu_resource_name", "gpu_request", "gpu_limit", "memory_bytes_request", "memory_bytes_limit", "yaml_pod_template_spec", "create_time", "creator", "update_time", "updater")
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TYPE_UNSPECIFIED: _ClassVar[Kernel.Type]
        TYPE_PYTHON: _ClassVar[Kernel.Type]
        TYPE_R: _ClassVar[Kernel.Type]
        TYPE_SPARK_PYTHON: _ClassVar[Kernel.Type]
        TYPE_SPARK_R: _ClassVar[Kernel.Type]
    TYPE_UNSPECIFIED: Kernel.Type
    TYPE_PYTHON: Kernel.Type
    TYPE_R: Kernel.Type
    TYPE_SPARK_PYTHON: Kernel.Type
    TYPE_SPARK_R: Kernel.Type
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    IMAGE_FIELD_NUMBER: _ClassVar[int]
    CPU_REQUEST_FIELD_NUMBER: _ClassVar[int]
    CPU_LIMIT_FIELD_NUMBER: _ClassVar[int]
    GPU_RESOURCE_NAME_FIELD_NUMBER: _ClassVar[int]
    GPU_REQUEST_FIELD_NUMBER: _ClassVar[int]
    GPU_LIMIT_FIELD_NUMBER: _ClassVar[int]
    MEMORY_BYTES_REQUEST_FIELD_NUMBER: _ClassVar[int]
    MEMORY_BYTES_LIMIT_FIELD_NUMBER: _ClassVar[int]
    YAML_POD_TEMPLATE_SPEC_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATER_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    type: Kernel.Type
    image: str
    cpu_request: int
    cpu_limit: int
    gpu_resource_name: str
    gpu_request: int
    gpu_limit: int
    memory_bytes_request: int
    memory_bytes_limit: int
    yaml_pod_template_spec: str
    create_time: _timestamp_pb2.Timestamp
    creator: str
    update_time: _timestamp_pb2.Timestamp
    updater: str
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., type: _Optional[_Union[Kernel.Type, str]] = ..., image: _Optional[str] = ..., cpu_request: _Optional[int] = ..., cpu_limit: _Optional[int] = ..., gpu_resource_name: _Optional[str] = ..., gpu_request: _Optional[int] = ..., gpu_limit: _Optional[int] = ..., memory_bytes_request: _Optional[int] = ..., memory_bytes_limit: _Optional[int] = ..., yaml_pod_template_spec: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., creator: _Optional[str] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., updater: _Optional[str] = ...) -> None: ...



from h2o_notebook.gen.ai.h2o.notebook.v1 import kernel_pb2 as _kernel_pb2
from google.api import annotations_pb2 as _annotations_pb2
from google.api import resource_pb2 as _resource_pb2
from google.api import client_pb2 as _client_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateKernelRequest(_message.Message):
    __slots__ = ("kernel", "kernel_id")
    KERNEL_FIELD_NUMBER: _ClassVar[int]
    KERNEL_ID_FIELD_NUMBER: _ClassVar[int]
    kernel: _kernel_pb2.Kernel
    kernel_id: str
    def __init__(self, kernel: _Optional[_Union[_kernel_pb2.Kernel, _Mapping]] = ..., kernel_id: _Optional[str] = ...) -> None: ...

class CreateKernelResponse(_message.Message):
    __slots__ = ("kernel",)
    KERNEL_FIELD_NUMBER: _ClassVar[int]
    kernel: _kernel_pb2.Kernel
    def __init__(self, kernel: _Optional[_Union[_kernel_pb2.Kernel, _Mapping]] = ...) -> None: ...

class GetKernelRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class GetKernelResponse(_message.Message):
    __slots__ = ("kernel",)
    KERNEL_FIELD_NUMBER: _ClassVar[int]
    kernel: _kernel_pb2.Kernel
    def __init__(self, kernel: _Optional[_Union[_kernel_pb2.Kernel, _Mapping]] = ...) -> None: ...

class ListKernelsRequest(_message.Message):
    __slots__ = ("page_size", "page_token")
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    page_size: int
    page_token: str
    def __init__(self, page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class ListKernelsResponse(_message.Message):
    __slots__ = ("kernels", "next_page_token")
    KERNELS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    kernels: _containers.RepeatedCompositeFieldContainer[_kernel_pb2.Kernel]
    next_page_token: str
    def __init__(self, kernels: _Optional[_Iterable[_Union[_kernel_pb2.Kernel, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class UpdateKernelRequest(_message.Message):
    __slots__ = ("kernel", "update_mask")
    KERNEL_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    kernel: _kernel_pb2.Kernel
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, kernel: _Optional[_Union[_kernel_pb2.Kernel, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class UpdateKernelResponse(_message.Message):
    __slots__ = ("kernel",)
    KERNEL_FIELD_NUMBER: _ClassVar[int]
    kernel: _kernel_pb2.Kernel
    def __init__(self, kernel: _Optional[_Union[_kernel_pb2.Kernel, _Mapping]] = ...) -> None: ...

class DeleteKernelRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class DeleteKernelResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from h2o_notebook.gen.ai.h2o.notebook.v1 import kernel_service_pb2 as ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2


class KernelServiceStub(object):
    """KernelService manages Kernels.
    (-- api-linter: core::0121::resource-must-support-get=disabled
    aip.dev/not-precedent: We support Get method --)
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateKernel = channel.unary_unary(
                '/ai.h2o.notebook.v1.KernelService/CreateKernel',
                request_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelRequest.SerializeToString,
                response_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelResponse.FromString,
                )
        self.GetKernel = channel.unary_unary(
                '/ai.h2o.notebook.v1.KernelService/GetKernel',
                request_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelRequest.SerializeToString,
                response_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelResponse.FromString,
                )
        self.ListKernels = channel.unary_unary(
                '/ai.h2o.notebook.v1.KernelService/ListKernels',
                request_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsRequest.SerializeToString,
                response_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsResponse.FromString,
                )
        self.UpdateKernel = channel.unary_unary(
                '/ai.h2o.notebook.v1.KernelService/UpdateKernel',
                request_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelRequest.SerializeToString,
                response_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelResponse.FromString,
                )
        self.DeleteKernel = channel.unary_unary(
                '/ai.h2o.notebook.v1.KernelService/DeleteKernel',
                request_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelRequest.SerializeToString,
                response_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelResponse.FromString,
                )


class KernelServiceServicer(object):
    """KernelService manages Kernels.
    (-- api-linter: core::0121::resource-must-support-get=disabled
    aip.dev/not-precedent: We support Get method --)
    """

    def CreateKernel(self, request, context):
        """Create a new Kernel.
        (-- api-linter: core::0133::http-uri-parent=disabled
        aip.dev/not-precedent: Kernel has no parent (it's a top-level resource) --)
        (-- api-linter: core::0133::method-signature=disabled
        aip.dev/not-precedent: Kernel has no parent (it's a top-level resource) --)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetKernel(self, request, context):
        """Return a specific Kernel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListKernels(self, request, context):
        """Return a collection of Kernels.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateKernel(self, request, context):
        """Update a Kernel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteKernel(self, request, context):
        """Delete a Kernel.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_KernelServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateKernel': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateKernel,
                    request_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelRequest.FromString,
                    response_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelResponse.SerializeToString,
            ),
            'GetKernel': grpc.unary_unary_rpc_method_handler(
                    servicer.GetKernel,
                    request_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelRequest.FromString,
                    response_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelResponse.SerializeToString,
            ),
            'ListKernels': grpc.unary_unary_rpc_method_handler(
                    servicer.ListKernels,
                    request_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsRequest.FromString,
                    response_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsResponse.SerializeToString,
            ),
            'UpdateKernel': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateKernel,
                    request_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelRequest.FromString,
                    response_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelResponse.SerializeToString,
            ),
            'DeleteKernel': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteKernel,
                    request_deserializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelRequest.FromString,
                    response_serializer=ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ai.h2o.notebook.v1.KernelService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class KernelService(object):
    """KernelService manages Kernels.
    (-- api-linter: core::0121::resource-must-support-get=disabled
    aip.dev/not-precedent: We support Get method --)
    """

    @staticmethod
    def CreateKernel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ai.h2o.notebook.v1.KernelService/CreateKernel',
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelRequest.SerializeToString,
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.CreateKernelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetKernel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ai.h2o.notebook.v1.KernelService/GetKernel',
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelRequest.SerializeToString,
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.GetKernelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListKernels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ai.h2o.notebook.v1.KernelService/ListKernels',
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsRequest.SerializeToString,
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.ListKernelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateKernel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ai.h2o.notebook.v1.KernelService/UpdateKernel',
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelRequest.SerializeToString,
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.UpdateKernelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteKernel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ai.h2o.notebook.v1.KernelService/DeleteKernel',
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelRequest.SerializeToString,
            ai_dot_h2o_dot_notebook_dot_v1_dot_kernel__service__pb2.DeleteKernelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from h2o_notebook.gen.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from h2o_notebook.gen.model.kernel_resource import KernelResource
from h2o_notebook.gen.model.protobuf_any import ProtobufAny
from h2o_notebook.gen.model.rpc_status import RpcStatus
from h2o_notebook.gen.model.v1_create_kernel_response import V1CreateKernelResponse
from h2o_notebook.gen.model.v1_get_kernel_response import V1GetKernelResponse
from h2o_notebook.gen.model.v1_kernel import V1Kernel
from h2o_notebook.gen.model.v1_kernel_type import V1KernelType
from h2o_notebook.gen.model.v1_list_kernels_response import V1ListKernelsResponse
from h2o_notebook.gen.model.v1_update_kernel_response import V1UpdateKernelResponse



# flake8: noqa

"""
    ai/h2o/notebook/v1/kernel.proto

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: version not set
    Generated by: https://openapi-generator.tech
"""


__version__ = "0.3.0"

# import ApiClient
from h2o_notebook.gen.api_client import ApiClient

# import Configuration
from h2o_notebook.gen.configuration import Configuration

# import exceptions
from h2o_notebook.gen.exceptions import OpenApiException
from h2o_notebook.gen.exceptions import ApiAttributeError
from h2o_notebook.gen.exceptions import ApiTypeError
from h2o_notebook.gen.exceptions import ApiValueError
from h2o_notebook.gen.exceptions import ApiKeyError
from h2o_notebook.gen.exceptions import ApiException



import pprint

from h2o_notebook.clients.kernel.kernel import from_api_object
from h2o_notebook.gen.model.v1_list_kernels_response import V1ListKernelsResponse


class KernelsPage:
    """Class represents a list of Kernels together with a next_page_token for listing all Kernels."""

    def __init__(self, list_api_response: V1ListKernelsResponse) -> None:
        api_objects = list_api_response.kernels
        self.kernels = []
        for api_kernel in api_objects:
            self.kernels.append(from_api_object(api_kernel))

        self.next_page_token = list_api_response.next_page_token

    def __repr__(self) -> str:
        return pprint.pformat(self.__dict__)



import http
import os

import pytest

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException


def test_get_kernel(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    yaml_spec = open(os.path.join(os.path.dirname(__file__), "pod_template_spec.yaml"),"r").read()

    kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_R,
        image="something",
        display_name="my first kernel",
        cpu_request=1,
        cpu_limit=2,
        gpu_resource_name="gpu-resource-name",
        gpu_request=3,
        gpu_limit=4,
        memory_bytes_request="5",
        memory_bytes_limit="6",
        yaml_pod_template_spec=yaml_spec,
    )

    k = kernel_client_super_admin.get_kernel(kernel_id="my-first-kernel")

    assert k.name == "kernels/my-first-kernel"
    assert k.kernel_id == "my-first-kernel"
    assert k.type == KernelType.TYPE_R
    assert k.display_name == "my first kernel"
    assert k.image == "something"
    assert k.cpu_request == 1
    assert k.cpu_limit == 2
    assert k.gpu_resource_name == "gpu-resource-name"
    assert k.gpu_request == 3
    assert k.gpu_limit == 4
    assert k.memory_bytes_request == "5"
    assert k.memory_bytes_limit == "6"
    assert k.yaml_pod_template_spec == yaml_spec
    assert k.create_time is not None
    assert k.creator is not None
    assert k.update_time is None
    assert k.updater is None

def test_get_kernel_not_found(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    with pytest.raises(CustomApiException) as exc:
        # Try to create Kernel with the same ID.
        kernel_client_super_admin.get_kernel(
            kernel_id="my-first-kernel",
        )

    assert exc.value.status == http.HTTPStatus.NOT_FOUND



import json

from h2o_notebook.gen import ApiException


class TimeoutException(Exception):
    """Exception raised when timeout is exceeded."""

    def __init__(self):
        super().__init__("Timeout exceeded")


class CustomApiException(ApiException):
    """ApiException with simplified error message.
    """

    def __init__(self, e: ApiException):
        self.status = e.status
        self.reason = e.reason
        self.body = e.body
        self.headers = e.headers

    def __str__(self):
        status_reason = f"{self.status} ({self.reason})"
        message = self.body

        if message is None:
            return status_reason

        try:
            body_json = json.loads(self.body)
            message = body_json["message"]
        except (ValueError, KeyError):
            pass

        return f"{status_reason}: {message}"



import pprint

from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.clients.kernel.type import (
    from_api_object as from_kernel_type_api_object,
)
from h2o_notebook.gen.model.kernel_resource import KernelResource
from h2o_notebook.gen.model.v1_kernel import V1Kernel


class Kernel:
    """Kernel object."""

    def __init__(
            self,
            name: str,
            display_name: str,
            type: KernelType,
            image: str,
            cpu_request: int,
            cpu_limit: int,
            gpu_resource_name: str,
            gpu_request: int,
            gpu_limit: int,
            memory_bytes_request: int,
            memory_bytes_limit: int,
            yaml_pod_template_spec: str,
            create_time: str,
            creator: str,
            update_time: str,
            updater: str,
    ):
        self.name = name
        self.display_name = display_name
        self.type = type
        self.image = image
        self.cpu_request = cpu_request
        self.cpu_limit = cpu_limit
        self.gpu_resource_name = gpu_resource_name
        self.gpu_request = gpu_request
        self.gpu_limit = gpu_limit
        self.memory_bytes_request = memory_bytes_request
        self.memory_bytes_limit = memory_bytes_limit
        self.yaml_pod_template_spec = yaml_pod_template_spec
        self.create_time = create_time
        self.creator = creator
        self.update_time = update_time
        self.updater = updater

        if name:
            self.kernel_id = self.name.split("/")[1]

    def __repr__(self) -> str:
        return pprint.pformat(self.__dict__)


    def to_api_object(self) -> KernelResource:
        return KernelResource(
            display_name = self.display_name,
            type = self.type.to_api_object(),
            image = self.image,
            cpu_request = self.cpu_request,
            cpu_limit = self.cpu_limit,
            gpu_resource_name = self.gpu_resource_name,
            gpu_request = self.gpu_request,
            gpu_limit = self.gpu_limit,
            memory_bytes_request = self.memory_bytes_request,
            memory_bytes_limit = self.memory_bytes_limit,
            yaml_pod_template_spec = self.yaml_pod_template_spec,
        )


def from_api_object(api_object: V1Kernel) -> Kernel:
    return Kernel(
        name = api_object.name,
        display_name = api_object.display_name,
        type = from_kernel_type_api_object(api_object.type),
        image = api_object.image,
        cpu_request = api_object.cpu_request,
        cpu_limit = api_object.cpu_limit,
        gpu_resource_name = api_object.gpu_resource_name,
        gpu_request = api_object.gpu_request,
        gpu_limit = api_object.gpu_limit,
        memory_bytes_request = api_object.memory_bytes_request,
        memory_bytes_limit = api_object.memory_bytes_limit,
        yaml_pod_template_spec = api_object.yaml_pod_template_spec,
        create_time = api_object.create_time,
        creator = api_object.creator,
        update_time = api_object.update_time,
        updater = api_object.updater,
    )


import http
import os

import pytest

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException


def test_delete_kernel(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    k = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    kernel_client_super_admin.delete_kernel(kernel_id=k.kernel_id)

    with pytest.raises(CustomApiException) as exc:
        kernel_client_super_admin.get_kernel(kernel_id=k.kernel_id)
    assert exc.value.status == http.HTTPStatus.NOT_FOUND

def test_delete_kernel_not_found(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    with pytest.raises(CustomApiException) as exc:
        kernel_client_super_admin.delete_kernel(kernel_id="not-found")
    assert exc.value.status == http.HTTPStatus.NOT_FOUND


from typing import Dict
from typing import List
from typing import Optional

from h2o_authn import TokenProvider

from h2o_notebook.clients.auth.token_api_client import TokenApiClient
from h2o_notebook.clients.kernel.kernel import Kernel
from h2o_notebook.clients.kernel.kernel import from_api_object
from h2o_notebook.clients.kernel.page import KernelsPage
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException
from h2o_notebook.gen import ApiException
from h2o_notebook.gen import Configuration
from h2o_notebook.gen.api.kernel_service_api import KernelServiceApi
from h2o_notebook.gen.model.v1_kernel import V1Kernel
from h2o_notebook.gen.model.v1_list_kernels_response import V1ListKernelsResponse


class KernelClient:
    """KernelClient manages Python kernels."""

    def __init__(
            self,
            server_url: str,
            token_provider: TokenProvider,
            verify_ssl: bool = True,
            ssl_ca_cert: Optional[str] = None,
    ):
        configuration = Configuration(host=server_url)
        configuration.verify_ssl = verify_ssl
        configuration.ssl_ca_cert = ssl_ca_cert

        with TokenApiClient(
                configuration, token_provider
        ) as api_client:
            self.api_instance = KernelServiceApi(api_client)

    def create_kernel(
            self,
            kernel_id: str,
            kernel_type: KernelType,
            image: str,
            display_name: str = "",
            cpu_request: int = 0,
            cpu_limit: int = 0,
            gpu_resource_name: str = "",
            gpu_request: int = 0,
            gpu_limit: int = 0,
            memory_bytes_request: str = "0",
            memory_bytes_limit: str = "0",
            yaml_pod_template_spec: str = "",
    ) -> Kernel:
        """Creates a kernel.

        Args:
            kernel_id (str): Kernel ID.
            kernel_type (KernelType): Kernel type.
            image (str): Kernel image.
            display_name (str): Kernel display name.
            cpu_request (int): CPU request.
            cpu_limit (int): CPU limit.
            gpu_resource_name (str): GPU resource name.
            gpu_request (int): GPU request.
            gpu_limit (int): GPU limit.
            memory_bytes_request (str): Memory request.
            memory_bytes_limit (str): Memory limit.
            yaml_pod_template_spec (str): YAML pod template spec.

        Returns:
            Kernel: Kernel object.
        """
        api_object = V1Kernel(
            type=kernel_type.to_api_object(),
            image=image,
            display_name=display_name,
            cpu_request=cpu_request,
            cpu_limit=cpu_limit,
            gpu_resource_name=gpu_resource_name,
            gpu_request=gpu_request,
            gpu_limit=gpu_limit,
            memory_bytes_request=memory_bytes_request,
            memory_bytes_limit=memory_bytes_limit,
            yaml_pod_template_spec=yaml_pod_template_spec,
        )
        created_api_object: V1Kernel

        try:
            created_api_object = self.api_instance.kernel_service_create_kernel(
                kernel_id=kernel_id,
                kernel=api_object,
            ).kernel
        except ApiException as e:
            raise CustomApiException(e)

        return from_api_object(api_object=created_api_object)

    def get_kernel(self, kernel_id: str) -> Kernel:
        """Returns a kernel.

        Args:
            kernel_id (str): Kernel ID.

        Returns:
            Kernel: Kernel object.
        """
        api_object: V1Kernel

        try:
            api_object = self.api_instance.kernel_service_get_kernel(
                name=f"kernels/{kernel_id}",
            ).kernel
        except ApiException as e:
            raise CustomApiException(e)

        return from_api_object(api_object=api_object)

    def list_kernels(
            self,
            page_size: int = 0,
            page_token: str = "",
    ) -> KernelsPage:
        """Lists kernels.

        Args:
            page_size (int): Maximum number of kernels to return in a response.
                If unspecified (or set to 0), at most 50 InternalDAIVersions will be returned.
                The maximum value is 1000; values above 1000 will be coerced to 1000.
            page_token (str): Page token.
                Leave unset to receive the initial page.
                To list any subsequent pages use the value of 'next_page_token' returned from the KernelsPage.

        Returns:
            KernelsPage: KernelsPage object.
        """
        list_response: V1ListKernelsResponse

        try:
            list_response = (
                self.api_instance.kernel_service_list_kernels(
                    page_size=page_size,
                    page_token=page_token,
                )
            )
        except ApiException as e:
            raise CustomApiException(e)

        return KernelsPage(list_response)

    def list_all_kernels(self) -> List[Kernel]:
        """
        List all kernels.

        Returns:
            List of Kernels
        """
        all_kernels: List[Kernel] = []
        next_page_token = ""
        while True:
            kernel_list = self.list_kernels(
                page_size=0,
                page_token=next_page_token,
            )
            all_kernels = all_kernels + kernel_list.kernels
            next_page_token = kernel_list.next_page_token
            if next_page_token == "":
                break

        return all_kernels

    def update_kernel(
            self,
            kernel: Kernel,
            update_mask: str = "*",
    ) -> Kernel:
        """Updates a kernel.

        Args:
            kernel (Kernel): Kernel object with to-be-updated values.
            update_mask (str): Comma separated paths referencing which fields to update.
                Update mask must be non-empty.
                Allowed field paths are: {"display_name", "type", "image", "cpu_request", "cpu_limit", "gpu_resource_name",
                "gpu_request", "gpu_limit", "memory_bytes_request", "memory_bytes_limit", "yaml_pod_template_spec"}.

        Returns:
            Kernel: Updated Kernel object.
        """
        updated_api_object: V1Kernel

        try:
            updated_api_object = self.api_instance.kernel_service_update_kernel(
                kernel_name=f"kernels/{kernel.kernel_id}",
                kernel=kernel.to_api_object(),
                update_mask=update_mask,
            ).kernel
        except ApiException as e:
            raise CustomApiException(e)

        return from_api_object(api_object=updated_api_object)

    def delete_kernel(self, kernel_id: str) -> None:
        """Deletes a kernel.

        Args:
            kernel_id (str): Kernel ID.
        """
        try:
            self.api_instance.kernel_service_delete_kernel(
                name=f"kernels/{kernel_id}",
            )
        except ApiException as e:
            raise CustomApiException(e)


import os
import shutil
import time
from typing import Any

import grpc
from enterprise_gateway.services.kernels.remotemanager import RemoteMappingKernelManager

from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2 import GetKernelRequest
from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2 import ListKernelsRequest
from h2o_notebook.gen.ai.h2o.notebook.v1.kernel_service_pb2_grpc import (
    KernelServiceStub,
)


class NotebookRemoteMappingKernelManager(RemoteMappingKernelManager):
    def __init__(self, **kwargs):
        self.log.info("NotebookRemoteMappingKernelManager.__init__")
        addr = os.getenv("H2O_NOTEBOOK_SERVER_GRPC_ADDR")
        if not addr:
            raise ValueError("H2O_NOTEBOOK_SERVER_GRPC_ADDR environment variable is not set")

        channel = grpc.insecure_channel(addr)
        self.notebook_client = KernelServiceStub(channel)

        # Authorize GRPC requests with Kubernetes service account token if not disabled (local testing)
        # TODO: Support projected service account tokens (the token may update, so handle accordingly)
        self.request_metadata = []
        if os.getenv("NOTEBOOK_ENABLE_GRPC_AUTH", 'True').lower() == "false":
            self.log.info("GRPC auth is disabled")
        else:
            path = os.getenv("KUBERNETES_SERVICE_ACCOUNT_TOKEN_FILE", "/var/run/secrets/kubernetes.io/serviceaccount/token")
            self.request_metadata.append(('authorization', 'Bearer ' + read_file(path)))

        # Try calling an endpoint to verify the connection
        self.notebook_client.ListKernels(request=ListKernelsRequest(), metadata=self.request_metadata)

        super().__init__(**kwargs)

    async def start_kernel(self, *args: list[Any] | None, **kwargs: dict[str, Any] | None):
        kernel_name = kwargs["kernel_name"]
        self.log.info("NotebookRemoteMappingKernelManager.start_kernel: kernel_name=%s, kwargs=%s", kernel_name, kwargs)
        kernel = self.notebook_client.GetKernel(
            request=GetKernelRequest(
                name=kernel_name,
            ),
            metadata=self.request_metadata
        ).kernel

        # This is where we can modify kernel-pod.yaml.j2 file with data from kernel
        # For now, just create copy of the default file
        kernel_dir = f"/tmp/{kernel.name}"
        shutil.copytree("/opt/h2oai/gateway/scripts", f"{kernel_dir}", dirs_exist_ok=True)

        # Artificially delay
        time.sleep(2)
        return await super().start_kernel(**kwargs)


def read_file(path: str) -> str:
    with open(path, "r") as f:
        return f.read()



import os

import h2o_authn
import pytest as pytest
from h2o_authn import TokenProvider

import h2o_notebook
from h2o_notebook.clients.kernel.client import KernelClient


@pytest.fixture(scope="session")
def session():
    return h2o_notebook.Session()

@pytest.fixture(scope="session")
def token_provider_user() -> TokenProvider:
    return h2o_authn.TokenProvider(
        refresh_token=os.getenv("PLATFORM_TOKEN_USER"),
        issuer_url=os.getenv("PLATFORM_OIDC_URL"),
        client_id=os.getenv("PLATFORM_OIDC_CLIENT_ID"),
    )

@pytest.fixture(scope="session")
def token_provider_super_admin() -> TokenProvider:
    return h2o_authn.TokenProvider(
        refresh_token=os.getenv("PLATFORM_TOKEN_SUPER_ADMIN"),
        issuer_url=os.getenv("PLATFORM_OIDC_URL"),
        client_id=os.getenv("PLATFORM_OIDC_CLIENT_ID"),
    )

@pytest.fixture(scope="session")
def kernel_client_user(token_provider_user: TokenProvider):
    return KernelClient(
        server_url=os.getenv("NOTEBOOK_SERVER_URL"),
        token_provider=token_provider_user,
        verify_ssl=False,
    )

@pytest.fixture(scope="session")
def kernel_client_super_admin(token_provider_super_admin: TokenProvider):
    return KernelClient(
        server_url=os.getenv("NOTEBOOK_SERVER_URL"),
        token_provider=token_provider_super_admin,
        verify_ssl=False,
    )

@pytest.fixture(scope="function")
def kernels_cleanup_after(kernel_client_super_admin: KernelClient):
    yield

    kernels = kernel_client_super_admin.list_all_kernels()
    for k in kernels:
        kernel_client_super_admin.delete_kernel(kernel_id=k.kernel_id)


# Script to insert kernels into the database [as is 1.12.2023]

# Prerequisites: kubectl port-forward deployment/notebook 8080:8080 --namespace=notebook-system-dev

import h2o_authn

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType

super_admin_token = "INSERT HERE YOUR SUPER ADMIN PLATFORM TOKEN"
tp = h2o_authn.TokenProvider(refresh_token=super_admin_token, issuer_url="https://auth.cloud-dev.h2o.ai/auth/realms/hac-dev", client_id="hac-platform-public")

c = KernelClient(
    server_url="http://localhost:8080",
    token_provider=tp,
    verify_ssl=False,
)


c.create_kernel(
    kernel_id="python",
    display_name="Python",
    kernel_type=KernelType.TYPE_PYTHON,
    image="gcr.io/vorvan/h2oai/h2o-kernel-py:0.3.4",
    memory_bytes_limit="12884901888",
    memory_bytes_request="12884901888",
)

c.create_kernel(
    kernel_id="python-gpu",
    display_name="Python [GPU]",
    kernel_type=KernelType.TYPE_PYTHON,
    image="gcr.io/vorvan/h2oai/h2o-kernel-py-gpu:0.3.4",
    gpu_limit=1,
    gpu_request=1,
    cpu_limit=4,
    cpu_request=1,
    memory_bytes_limit="8589934592",
)

c.create_kernel(
    kernel_id="python-l",
    display_name="Python [L]",
    kernel_type=KernelType.TYPE_PYTHON,
    image="gcr.io/vorvan/h2oai/h2o-kernel-py:0.3.4",
    memory_bytes_limit="30064771072",
    memory_bytes_request="30064771072",
)

c.create_kernel(
    kernel_id="python-xl",
    display_name="Python [XL]",
    kernel_type=KernelType.TYPE_PYTHON,
    image="gcr.io/vorvan/h2oai/h2o-kernel-py:0.3.4",
    memory_bytes_limit="64424509440",
    memory_bytes_request="64424509440",
)

c.create_kernel(
    kernel_id="r",
    display_name="R",
    kernel_type=KernelType.TYPE_R,
    image="gcr.io/vorvan/h2oai/h2o-kernel-r:0.3.4",
    memory_bytes_limit="64424509440",
    memory_bytes_request="64424509440",
)

c.create_kernel(
    kernel_id="r-l",
    display_name="R [L]",
    kernel_type=KernelType.TYPE_R,
    image="gcr.io/vorvan/h2oai/h2o-kernel-r:0.3.4",
    memory_bytes_limit="173946175488",
    memory_bytes_request="173946175488",
)

c.create_kernel(
    kernel_id="python-spark",
    display_name="Spark - Python",
    kernel_type=KernelType.TYPE_SPARK_PYTHON,
    image="gcr.io/vorvan/h2oai/h2o-kernel-pyspark:0.3.4",
    memory_bytes_limit="12884901888",
    memory_bytes_request="12884901888",
)

c.create_kernel(
    kernel_id="r-spark",
    display_name="Spark - R",
    kernel_type=KernelType.TYPE_SPARK_R,
    image="gcr.io/vorvan/h2oai/h2o-kernel-sparkr:0.3.4",
    memory_bytes_limit="12884901888",
    memory_bytes_request="12884901888",
)

print(c.list_kernels())


import http
import os

import pytest

from h2o_notebook.clients.kernel.client import KernelClient
from h2o_notebook.clients.kernel.type import KernelType
from h2o_notebook.exception import CustomApiException


@pytest.mark.parametrize(
    "mask",
    [
        "unknown",
        "invalid character",
        "gpu, *",
        " ",
    ],
)
def test_update_mask_validation(kernel_client_super_admin: KernelClient, mask, kernels_cleanup_after):
    k = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    k.display_name = "Changed Smokerinho"

    with pytest.raises(CustomApiException) as exc:
        kernel_client_super_admin.update_kernel(kernel=k, update_mask=mask)
    assert exc.value.status == http.HTTPStatus.BAD_REQUEST


def test_update(kernel_client_super_admin: KernelClient, kernels_cleanup_after):
    original = kernel_client_super_admin.create_kernel(
        kernel_id="my-first-kernel",
        kernel_type=KernelType.TYPE_PYTHON,
        image="something",
    )

    original.gpu_limit = 1
    original.cpu_request = 2
    original.memory_bytes_request = "3"
    original.storage_bytes_limit = "4"

    # Update profile with update_mask.
    updated = kernel_client_super_admin.update_kernel(original, update_mask="cpu_request,memory_bytes_request")

    assert updated.cpu_request == 2
    assert updated.memory_bytes_request == "3"
    assert updated.gpu_limit == 0

    # Update profile without update_mask.
    updated = kernel_client_super_admin.update_kernel(original)

    assert updated.gpu_limit == 1